/*
        Defintions of various messages seen on the ethernet.

	Any changes or bugs contact me at miller@encompasserve.org

 COPYRIGHT NOTICE

 This software is COPYRIGHT  1997,2005, Ian Miller. 

 This software released under the license described in aaareadme.txt

 DISCLAIMER

 This software is provided "AS IS". The author makes no representations or
 warranties with respect to the software and specifically disclaim any implied
 warranties of merchantability or fitness for any particular purpose.

History	
=======

26-MAR-1997,2005 I.Miller  V1.0-000	Split from other files.
*/

#if __DECC
#pragma member_alignment save
#pragma nomember_alignment
#endif

/* DECnet IV message structures */
/* DECnet IV routing layer control flag - control packet        */
typedef struct
{
        unsigned ctrl:1;        /* = 1 indicates control packet */
        unsigned type:3;        /* message type                 */
        unsigned reserved:3;
        unsigned pad_field:1;   /* = 0 no padding               */
} DECNETIV_FLAGS;

typedef struct
{
        unsigned char version;  /* = 2                          */
        unsigned char eco_no;   /* ECO Number                   */
        unsigned char ueco_no;  /* user ECO number              */
} DECNETIV_VERSION;

typedef struct
{
        unsigned type:2;        /* node type */
        unsigned verreq:1;      /* verification required flag   */
        unsigned reject:1;      
        unsigned verfailed:1;   /* verification failed flag     */
        unsigned nomulticast:1; /* No Multicast Traffic flag    */
        unsigned blockingreq:1; /* blocking requested flag      */
        unsigned reserved:1;  
} DECNETIV_IINFO;

/* structure of a DECnet IV node ID (address)     */
typedef struct
{
        unsigned node:10;       /* node number within area 1-1023       */
        unsigned area:6;        /* area number 1-63 (0 means this area) */
} DECNETIV_ID;

/* DECnet IV ethernet endnode hello message     */
typedef struct
{
        DECNETIV_FLAGS  flags;          /* with type == 6               */
        DECNETIV_VERSION version;
        unsigned char id[6];            /* ID of transmitting node      */
        DECNETIV_IINFO info;            /* type = 3 end node            */
        unsigned short blksize;         /* max data link receive blk size */
        unsigned char area;             /* reserved                     */
        unsigned char seed[8];          /* verification seed            */
        unsigned char neighbor[6];      /* ID of designated router      */
        unsigned short timer;           /* hello timer in seconds       */
        unsigned char mpd;              /* reserved                     */
        unsigned char data_len;         /* test data length             */
        unsigned char data[128];        /* up to 128 bytes of 252 octal.*/
} DECNETIV_EHELLO;

/* DECnet IV ethernet router hello message     */
typedef struct
{
        DECNETIV_FLAGS  flags;          /* with type == 5               */
        DECNETIV_VERSION version;
        unsigned char id[6];            /* ID of transmitting node      */
        DECNETIV_IINFO info;            /* type = 2 L1 rtr,1 = L2 rtr   */
        unsigned short blksize;         /* max data link receive blk size */
        unsigned char priority;         /* router's priority            */
        unsigned char area;             /* reserved                     */
        unsigned short timer;           /* Hello timer in seconds       */
        unsigned char mpd;              /* reserved                     */
        unsigned char e_list_len;       /* logical ethernet list length */
        unsigned char name[7];          /* logical ethernet name - reserved */
        unsigned char rs_list_len;      /* router stat list length      */
        struct
        {
                unsigned char router[6];/* router ID                    */
                struct
                {
                        unsigned priority:7;    /* routers  priority    */
                        unsigned state:1;       /* 1 = means known 2-way*/
                } pristate;
        } rs_list[33];                  /* router/state list            */
} DECNETIV_RHELLO;

/* DECNET IV LEVEL 1 routing message    */
/* consists this header,1 or more segments and a check sum      */
typedef struct
{
        DECNETIV_FLAGS  flags;          /* with type == 3               */
        unsigned short srcnode;         /*  source node DECnet address  */
        unsigned char res[1];           /* reserved                     */
} DECNETIV_RLEVEL1_HDR;
/* a segment is a two byte count, a 2 byte ID, followed by count rtginfo's*/
/* the first rtginfo is for start_id, the seconds for start_id + 1 etc    */
typedef struct
{
        unsigned short count;           /* number of IDs in this segment*/
        unsigned short start_id;        /* ID - DECnet node addr with area=0 */
} DECNETIV_RLEVEL1_SEG;
typedef struct
{
        unsigned cost:10;       /* 1023 means not reachable     */
        unsigned hops:5;        /* 31 means not reachable       */
        unsigned unused:1;      /* always 0                     */
} DECNETIV_RLEVEL1_RTGINFO;
#define DECNETIV_INFC 1023   
#define DECNETIV_INFH 31

/* DECNET IV LEVEL 2 routing message    */
/* consists this header,1 or more segments and a check sum      */
typedef struct
{
        DECNETIV_FLAGS  flags;          /* with type == 4               */
        unsigned short srcnode;       
        unsigned char res[1];           /* reserved                     */
} DECNETIV_RLEVEL2_HDR;
/* a segment is a two byte count,2 byte area, followed by count rtginfo's  */
/* the first rtginfo is for start_area, the seconds for start_area + 1 etc */
typedef struct
{
        unsigned short count;           /* number of AREAs in this segment*/
        unsigned short start_area;      /* area number for for 1st seg */
} DECNETIV_RLEVEL2_SEG;
typedef struct
{
        unsigned cost:10;
        unsigned hops:5;
        unsigned unused:1;      /* always 0     */
} DECNETIV_RLEVEL2_RTGINFO;

/* IP related definitions       */
/* IP V4 packet header          */
typedef struct
{
	unsigned int version:4;	/* IP version 				*/
	unsigned int ihl:4;	/* internet header length in 32 bit words */
	unsigned int tos:8;	/* type of service			*/
	unsigned int tlength:16;/* length of the IP datagram		*/
	unsigned int id:16;	/* datagram id				*/
	unsigned int flags:3;	/* flags				*/
	unsigned int fragment_offset:13;	/* which fragment is this */
	unsigned int ttl:8;	/* time to live				*/
	unsigned int protocol:8;/* protocol				*/
	unsigned int header_checksum:16;	/* header checksum	*/
	unsigned char source[4];/* source address			*/
	unsigned char destination[4];	/* destination address		*/
        unsigned char options[3];
        unsigned char pad;
} IP_HEADER;

#define IP_ICMP		  1
#define IP_IGMP		  2
#define IP_TCP 		  6
#define IP_UDP		 17

/* UDP header (follows IP header when protocol is 17 	*/
typedef struct
{
	unsigned short sport;	/* source port		*/
	unsigned short dport;	/* destination port	*/
	unsigned short length;	/* length including hdr */
	unsigned short checksum;/* checksum		*/
} UDP_HEADER;

/* UDP port numbers     */
#define UDP_PORT_BOOTP_SERVER	67	/* bootp/DHCP server - request	*/
#define UDP_PORT_BOOTP_CLIENT	68	/* bootp/DHCP server - response	*/
#define UDP_PORT_NETBIOS_NAME   137
#define UDP_PORT_NETBIOS_DGRAM  138
#define UDP_PORT_NETBIOS        139
#define UDP_PORT_IPX            213
#define UDP_PORT_NETWAREIP      398
#define UDP_PORT_ROUTE          520

/* BOOTP/DHCP header    */
typedef struct
{
        unsigned char op;       /* BOOTREQUEST/BOOTREPLY                */
        unsigned char htype;    /* hardware address type = ARP_HRD_ETH  */
        unsigned char hlen;     /* hardware address len =  ARP_HLN_ETH  */
        unsigned char hops;     /* number of hops for booting over gateways */
        unsigned long xid;      /* transaction id                       */
        unsigned short secs;    /* seconds since client started         */
        struct
        {
                unsigned broadcast:1;   /* set if reply should be broadcast*/
                unsigned unused:15;
        } flags;
        unsigned char ciaddr[4];/* client IP address - zero if not known*/
        unsigned char yiaddr[4];/* 'your' (client) IP address (from server)*/
        unsigned char siaddr[4];/* server IP address                    */
        unsigned char giaddr[4];/* gateway IP address                   */
        unsigned char chaddr[16];/* client hardware address             */
        char sname[64];         /* server name (optional) - asciz       */
        char file[128];         /* filename (in boot reply) - asciz     */
        unsigned char options[312]; /* options - variable length        */
} BOOTP_HEADER;
/* values of op field   */
#define BOOTREQUEST   1         /* request from client                  */
#define BOOTREPLY     2         /* reply from server                    */


/* ICMP header (follows IP header when protocol is 1)   */
/* often followed by IP header and 8 bytes of original  */
/* message that caused this ICMP message.               */
typedef struct
{
        unsigned char icmp_type;
        unsigned char icmp_code;       
        unsigned short icmp_checksum;
        union
        {
                unsigned char unused[4];
                struct
                {
                        unsigned char pointer;  /* byte offset in hdr */
                        unsigned char unused[3];
                } param_prob;
                unsigned char gateway[4];       /* IP address of gateway     */
                struct
                {
                        unsigned short id;
                        unsigned short sequence;
                } echo;
                struct
                {
                        unsigned char num_addr; /* number of address entries */
                        unsigned char addr_size;/* addr entry size in longw. */
                        unsigned char lifetime; /* addr lifetime in second   */
                        struct
                        {
                                char addr[4];
                                unsigned long preference;
                        } entry[1];
                } router_adv;
        } u;
} ICMP_HEADER;
#define ICMP_TYPE_ECHO_REQUEST  0       /* echo request (ping)          */
#define ICMP_TYPE_DUNREACH      3       /* destination unreachable      */
#define IMCP_TYPE_SQUENCH       4       /* source quench                */
#define ICMP_TYPE_REDIRECT      5       /* redirect                     */
#define ICMP_TYPE_ECHO_REPLY    8       /* echo reply (ping response)   */
#define ICMP_TYPE_ROUTER_ADV    9       /* router advertisment          */
#define ICMP_TYPE_ROUTER_SOL    10      /* Router Solicitation          */
#define ICMP_TYPE_TOOOLD        11      /* time exceeded                */
#define ICMP_TYPE_PPROBLEM      12      /* parameter problem            */
#define ICMP_TYPE_TIMESTAMP     13      /* timestamp request            */
#define ICMP_TYPE_TIMESTAMP_REP 14      /* timestamp reply              */
#define ICMP_TYPE_INFO_REQUEST  15      /* information request          */
#define ICMP_TYPE_INFO_REPLY    16      /* information reply            */

/* IGMP header - for V1, V2 queries and reports and V3 queries */
typedef struct {
        unsigned char type;     /* IGMP message type            */
        unsigned char maxrespcode;
        unsigned short checksum;
        unsigned char gaddr[4]; /* group address                */
	unsigned reserved:4;	/* unused			*/
	unsigned s:1;		/* suppress router side processing flag */
	unsigned qrv:3;		/* Querier's Robustness Variable*/
	unsigned char qqic;     /* Querier's Query Interval Code*/
	unsigned short n;	/* number of sources		*/
	unsigned char saddr[4][366]; /* source addresses	*/
} IGMP_HEADER;

/* IGMP V3 report group packet	*/
typedef struct
{
	unsigned char type;	/* record type			*/
	unsigned char auxdatalen;
	unsigned short nsources;
	unsigned char gaddr[4];	/* group multicast address	*/
	unsigned char srcaddr[4][1];
} IGMP_GROUP;

/* IGMP V3 report packet	*/
typedef struct
{
	unsigned char type;	/* message type 0x22		*/
	unsigned char reserved;	/* ununsed part			*/
	unsigned short checksum;
	unsigned short reserved2;
	unsigned short ngroups;	/* number of group records	*/
	IGMP_GROUP groups[1];	/* one or more group records	*/
} IGMP_REPORT_V3;

/* types defined in RFC 3376	*/
#define IGMP_TYPE_QUERY 0x11    /* Host membership query        */
#define IGMP_TYPE_REPORT_V1 0x12 /* host membership report - V1	RFC1112 */
#define IGMP_TYPE_REPORT_V2 0x16 /* host membership report - V2 RFC2236 */ 
#define IGMP_TYPE_REPORT_V3 0x22 /* host membership report - V3 RFC3376 */
#define IGMP_TYPE_LEAVE_GROUP 0x17 /* leave group - V2 RFC2236 	*/

/* RIP header (follows UDP header) including RIP-2 fields       */
typedef struct
{
        unsigned char command;
        unsigned char version;
        unsigned short rd;      /* routing domain (V2 only)     */
} RIP_HEADER;

/* one or more RIP entries follow the RIP header        */
typedef struct
{
        unsigned short afi;     /* address family identifier            */
        unsigned short tag;     /* route tag (V2 only)                  */
        unsigned char addr[4];  /* address                              */
        unsigned char mask[4];  /* subnet mask (V2 only)                */
        unsigned char next_hop[4]; /* next hop IP address (V2 only)     */
        long metric;   		/* routing metric value                 */
} RIP_ENTRY;
/* if AFI is 0xFFFF then the entry is a Authentication entry (V2 only)  */
typedef struct
{
        unsigned short flag;    /* 0xFFFF to indicate Authentication    */
        unsigned short type;    /*  Authentication type                 */
        unsigned char authentication[16];
} RIP_AUTH_ENTRY;

/* IPX protocol (Novell)        */
#define IPX_NODE_LEN	6
typedef struct
{
	unsigned long net;
	unsigned char  node[IPX_NODE_LEN]; 
	unsigned short sock;
} IPX_ADDRESS;

#define IPX_BROADCAST_NODE	"\377\377\377\377\377\377"
#define IPX_THIS_NODE           "\0\0\0\0\0\0"
#define IPX_NO_CHECKSUM	0xFFFF
/* IPX message types    */
#define IPX_TYPE_UNKNOWN	0x00
#define IPX_TYPE_RIP		0x01	/* Routing Information Packet   */
#define IPX_TYPE_SAP		0x04	/* Service Advertising Packet   */
#define IPX_TYPE_SPX		0x05	/* Sequenced Packed eXchange    */
#define IPX_TYPE_NCP		0x11	/* NetWare Core Protocol        */
#define IPX_TYPE_PPROP		0x14	/* propagated packet (NetBIOS)  */

/* IPX reserved destination socket numbers                      */
/* can be used to determine message type if type field is zero  */
#define IPX_SOCK_NCP            0X0451  /* NetWare Core Protocol        */
#define IPX_SOCK_SAP            0x0452  /* Service Advertising Packet   */
#define IPX_SOCK_RIP            0x0453  /* Routing Information Packet   */
#define IPX_SOCK_NETBIOS        0x0455  /* Novell NetBIOS process       */

#define IPX_MAX_HOP             16      /* this many hops and its unreacable */
typedef struct
{
	unsigned short	ipx_checksum;   /* checksum or FFFF             */
	unsigned short  ipx_pktsize;    /* packet size including header */
	unsigned char   ipx_tctrl;      /* number of hops               */
	unsigned char   ipx_type;       /* message type                 */
	IPX_ADDRESS	ipx_dest;       /* destination address          */
	IPX_ADDRESS	ipx_source;     /* source address               */
} IPX_HEADER;

/*      SPX header      */
typedef struct 
{
	unsigned char cctl;	
#define CCTL_SPXII_XHD	0x01	/* SPX2 extended header */
#define CCTL_SPX_UNKNOWN 0x02	/* Unknown (unused ??) */
#define CCTL_SPXII_NEG	0x04	/* Negotiate size */
#define CCTL_SPXII	0x08	/* Set for SPX2 */
#define CCTL_EOM	0x10	/* End of message marker */
#define CCTL_URG	0x20	/* Urgent marker in SPP (not used in SPX?) */
#define CCTL_ACK	0x40	/* Send me an ACK */
#define CCTL_CTL	0x80	/* Control message */
	unsigned char dtype;
#define SPX_DTYPE_ECONN	0xFE	/* Finished */
#define SPX_DTYPE_ECACK	0xFF	/* Ok */
	unsigned short sconn;	/* Connection ID */
	unsigned short dconn;	/* Connection ID */
	unsigned short sequence;
	unsigned short ackseq;
	unsigned short allocseq;
} SPX_HEADER;

/* RIP message  */
typedef struct
{
        unsigned long network;
        unsigned short hops;
        unsigned short ticks;
} IPX_RIP_ENTRY;
#define IPX_RIP_MAX_NETWORKS    50      /* max network entries in one RIP msg*/
typedef struct
{
        unsigned short operation;
        IPX_RIP_ENTRY networks[IPX_RIP_MAX_NETWORKS];
} IPX_RIP_MESSAGE;
#define IPX_RIP_OP_REQUEST      1
#define IPX_RIP_OP_RESPONSE     2

/* SAP message  */
typedef struct
{
        unsigned short type;            /* service type */
        char name[48];                  /* server name (null terminated)    */
        unsigned long network;          /* network number                   */
        unsigned char addr[6];          /* address of server                */
        unsigned short socket;          /* socket number for this service   */
        unsigned short hops;            /* hops to server                   */
} IPX_SAP_ENTRY;
#define IPX_SAP_MAX_SERVICE_ENTRIES     7 /* max entries in one SAP message   */
typedef struct
{
        unsigned short operation;
        IPX_SAP_ENTRY entries[IPX_SAP_MAX_SERVICE_ENTRIES];
} IPX_SAP_MESSAGE;
/* IPX SAP operation            */
#define IPX_SAP_OP_REQUEST      1
#define IPX_SAP_OP_RESPONSE     2
#define IPX_SAP_OP_GNS_REQUEST  3       /* get nearest server request   */
#define IPX_SAP_OP_GNS_RESPONSE 4       /* get nearest server response  */

/* IPX SAP service types        */
#define IPX_SAP_TYPE_UNKNOWN    0X0000          /* unknown              */
#define IPX_SAP_TYPE_PRINTQ     0x0003          /* print queue          */
#define IPX_SAP_TYPE_FSERVER    0x0004          /* File server          */
#define IPX_SAP_TYPE_JSERVER    0x0005          /* Job server           */
#define IPX_SAP_TYPE_PSERVER    0x0007          /* print server         */
#define IPX_SAP_TYPE_ASERVER    0x0009          /* Archive server       */
#define IPX_SAP_TYPE_RBSERVER   0x0024          /* Remote Bridge server */
#define IPX_SAP_TYPE_APSERVER   0x0047          /* Advertising Print Svr*/

/* ARP messages (for IP over ethernet)  */
#define ARP_REQUEST     0x0001
#define ARP_REPLY       0x0002
#define RARP_REQUEST    0x0003
#define RARP_REPLY      0x0004
#define ARP_HRD_ETH     0x0001
#define ARP_PRO_IP      0x0800
#define ARP_HLN_ETH     6
#define ARP_PLN_IP      4

typedef struct
{
        unsigned short ar_hrd;  /* hardware address space       */
        unsigned short ar_pro;  /* protocol address space       */
        unsigned char ar_hln;   /* hardware address length (=6) */
        unsigned char ar_pln;   /* protocol address length (=4) */
        unsigned short ar_op;   /* opcode (request,reply)       */
        unsigned char ar_sha[6];/* hardware address of sender   */
        unsigned char ar_spa[4];/* protocol address of sender   */
        unsigned char ar_tha[6];/* hardware address of target   */
        unsigned char ar_tpa[4];/* protocol address of target   */
} ARP_MESSAGE;

/* LAVC announcement    */
typedef struct
{
        unsigned char mca[6];   /* cluster multicast address    */
        unsigned short gno;     /* cluster group code no.       */
        unsigned char addr[6];  /* cluster node address         */
        char u1[11];            /* unknown                      */
        char name[6];           /* node name                    */
        char u2[57];            /* unknown                      */
        unsigned char lan_dn_len; /* lan device name length     */
        char lan_device_name[3]; /* lan device name             */
        char u3[20];            /* unknown                      */
} LAVC_ANNOUNCEMENT;

/*
        Definitions for LAT messages
*/

/* message types        */
#define LAT_C_RUN_MSG           0
#define LAT_C_START_MSG         1
#define LAT_C_STOP_MSG          2
#define LAT_C_SERVICE_MSG       10
#define LAT_C_COMMAND_MSG       12
#define LAT_C_STATUS_MSG        13
#define LAT_C_SOLICIT_INFO      14
#define LAT_C_RESPONSE_INFO     15

/* service annoucement message header */
typedef struct
{
        struct
        {
                unsigned rrf:1;
                unsigned master:1;
                unsigned type:6;        /* message type */
        } msg_type;
        unsigned char service_circ_timer;
        unsigned char protocol_version_hi;
        unsigned char protocol_version_lo;
        unsigned char protocol_version_cur;
        unsigned char protocol_version_eco;
        unsigned char msg_incarnation;
        unsigned char change_flags;
        unsigned short rcv_frame_size;
        unsigned char node_mulicast_timer;
        unsigned char node_status;
} LAT_SERVICE_HDR;

/* a service announcement contists of a sequence of counted strings     */
/* starting after the header                                            */
/* Each has a 1 byte length (may be 0) and then the data if len not 0.  */
/*      service group bitmask                                           */
/*      node name                                                       */
/*      node description                                                */
/* followed by 1 byte number of service announcement each of which      */
/* consists of                                                          */
/*      1 byte service rating                                           */
/*      service name (one byte length then ascii chars)                 */
/*      service description (one byte length then ascii chars - optional)*/

/* Local Area System Transport  */
/* should have ethernet protocol type = 80-41 and annoucements sent to
	multicast address 08-00-2b-80-41. I only seem to pick these up
	by matching on the multicast address. There is something strange
	about the frame header so they don't seem to match otherwise

        Type 5 are the node annoucements.
        Type 6 are broadcast to the multicast addr and appear to be requests
                for servers to respond with service descriptions.
        Message type 7 appear to be service descriptions sent to nodes
                who sent a message type 6.
        
*/
typedef struct
{
        unsigned long type;     /* message type         	*/
        unsigned char addr[6];  /* node hardware adddress       */
        unsigned char u1[9];    /* ? */
        unsigned char namelen;  /* node name length     	*/
        char name[8];           /* node name            	*/
        unsigned char u2[18];   /* ? */
        unsigned short id;      /* node id (low byte first)     */
        unsigned char u3[22];   /* ? */
} LAST_HDR;

#define LAST_TYPE_SERV_ANN      0x0005  /* server annoucement           */
#define LAST_TYPE_SERV_REQ      0x0006  /* request for servers          */
#define LAST_TYPE_SERV_RESP     0x0007  /* server response to type 6    */
#define LAST_TYPE_DISK_REQ      0x0800  /* request for disk block       */
#define LAST_TYPE_DISK_RESP     0X0400  /* response with disk block     */

/* DNS message (also used by NetBIOS name service)      */
/* fixed header         */
typedef struct {
	unsigned short	id;		/* query identification number */
	unsigned	rd :1;		/* recursion desired */
	unsigned	tc :1;		/* truncated message */
	unsigned	aa :1;		/* authoritive answer */
	unsigned	opcode :4;	/* purpose of message */
	unsigned	qr :1;		/* response flag */
	unsigned	rcode :4;	/* response code */
	unsigned	cd: 1;		/* checking disabled by resolver */
	unsigned	ad: 1;		/* authentic data from named */
	unsigned	unused :1;	/* unused bits (MBZ as of 4.9.3a3) */
	unsigned	ra :1;		/* recursion available */
	unsigned short	qdcount ;	/* number of question entries */
	unsigned short	ancount ;	/* number of answer entries */
	unsigned short	nscount ;	/* number of authority entries */
	unsigned short	arcount ;	/* number of resource entries */
} NAME_HEADER;
/* operation codes      */
#define NAME_OP_QUERY		0x0	/* standard query */
#define NAME_OP_IQUERY		0x1	/* inverse query */
#define NAME_OP_STATUS		0x2	/* nameserver status query */
#define NAME_OP_NS_NOTIFY_OP	0x4	/* notify secondary of SOA change */
#define NAME_OP_REGISTRATION    0x5     /* NetBIOS registration         */
#define NAME_OP_RELEASE         0x6     /* NetBIOS release              */
#define NAME_OP_WACK            0x7     /* NetBIOS Wait for ACK response*/
#define NAME_OP_REFRESH         0x8     /* NetBIOS refresh              */

/* rcode values */
#define NAME_OP_RCODE_FMT_ERR   0x1     /* request was invalidly formatted */
#define NAME_OP_RCODE_SRV_ERR   0x2     /* server error                 */
#define NAME_OP_RCODE_IMP_ERR   0x4     /* unsupported request          */
#define NAME_OP_RCODE_RFS_ERR   0x5     /* request refused              */
#define NAME_OP_RCODE_ACT_ERR   0x6     /* active error - name owned by other */
#define NAME_OP_RCODE_CFT_ERR   0x7     /* conflict error               */
/* class        */
#define NAME_CLASS_IN           0x01
#define NAME_CLASS_CHAOS        0x03
#define NAME_CLASS_HESIOD       0x04
#define NAME_CLASS_ANY          0xff

/* type		*/
#define NAME_TYPE_A		0x01	/* IP address RR		*/
#define NAME_TYPE_NS		0x02	/* name server RR		*/
#define NAME_TYPE_CNAME         0x05    /* CNAME                        */
#define NAME_TYPE_SOA           0x06    /* SOA                          */
#define NAME_TYPE_NULL		0x0A	/* NULL RR (used by WACK)	*/
#define NAME_TYPE_WKS           0x0B    /* Well Known Service           */
#define NAME_TYPE_PTR           0x0C    /* domain name pointer          */
#define NAME_TYPE_MX            0x0F    /* mail exchange                */
#define NAME_TYPE_NB		0x20	/* NetBIOS name	service RR	*/
#define NAME_TYPE_NBSTAT	0x21	/* NetBIOS node status RR 	*/

#define NAME_INDIR_FLAG         0xc0    /* indirect name flag mask     	*/

/* NetBIOS RR RDATA - NB_FLAGS  */
typedef struct
{
        unsigned reserved:13;
        unsigned ont:2;         /* owner node type      */
        unsigned group:1;       /* 1 if group name      */
} NB_FLAGS;
/* NetBIOS node types   */
#define NAME_NB_NODE_TYPE_B     0x00
#define NAME_NB_NODE_TYPE_P     0x01
#define NAME_NB_NODE_TYPE_M     0x10
#define NAME_NB_NODE_TYPE_R     0x11    /* reserved     */

/* NetBIOS RR RDATA - NB_ADDRESS  */
typedef struct
{
        unsigned char addr[4];          /* IP address of names owner    */
} NB_ADDRESS;

/* SMB header   */
typedef struct {
    unsigned long LowPart;
    long HighPart;
} LARGE_INTEGER;                        /* 64 bits of data      */

typedef struct  {
    unsigned char Protocol[4];          /* Contains 0xFF,'SMB'  */
    unsigned char Command;              /* Command code         */
    union {
        struct {
            unsigned char ErrorClass;   /* Error class          */
            unsigned char Reserved;     /* Reserved for future use      */
            unsigned short Error;       /* Error code                   */
        } DosError;
        unsigned long Status;           /* 32-bit error code            */
    } Status;
    unsigned char Flags;                /* Flags                        */
    unsigned short Flags2;              /* More flags                   */
    union {
        unsigned short Pad[6];          /* Ensure section is 12 bytes long */
        struct {
            unsigned short PidHigh;     /* High part of PID             */
            unsigned long  Unused;      /* Not used                     */
            unsigned long  Unused2;
    } Extra;
    } x;
    unsigned short Tid;                 /* Tree identifier              */
    unsigned short Pid;                 /* Caller's process id          */
    unsigned short Uid;                 /*Unauthenticated user id       */
    unsigned short Mid;                 /* multiplex id                 */
    unsigned char  WordCount;           /* Count of parameter words     */
#if 0
    unsigned short ParameterWords[ WordCount ];  /* The parameter words */
    unsigned short ByteCount;           /* Count of bytes               */
    unsigned char  Buffer[ ByteCount ]; /* The bytes                    */
#endif
} SMB_HEADER;

#if __DECC
#pragma member_alignment restore
#endif
